/*
 * Copyright (C) 2005-2009 Makoto Sato. All Rights Reserved.
 * Copyright (C) 2011 ObjectFanatics, Ltd. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.objectfanatics.assertion.mojo;

import jp.objectfanatics.assertion.constraints.NotEmpty;
import jp.objectfanatics.assertion.weaver.api.core.exception.IllegalUseOfConstraintAnnotationException;
import jp.objectfanatics.assertion.weaver.api.core.exception.NoWeavingNeededException;
import jp.objectfanatics.assertion.weaver.api.core.exception.NotClassFileException;
import jp.objectfanatics.assertion.weaver.impl.core.ClassFolderWeaver;
import jp.objectfanatics.assertion.weaver.impl.core.ClassFolderWeaver.WeavingProgressListener;
import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;

import java.io.File;
import java.io.IOException;

// checked 20110331

/**
 * TODO:2011-03-23 javadoc beyondseeker
 * reference
 * <ul>
 * <li>http://maven.apache.org/plugin-developers/index.html
 * <li>http://maven.apache.org/guides/introduction/introduction-to-the-lifecycle.html
 * </ul
 *
 * @author Makoto Sato
 * @author Hirokatsu Endo
 * @requiresDependencyResolution
 */
abstract public class AbstractWeavingMojo extends AbstractMojo {

    // checked 20110331
    /**
     * @parameter expression="${project.build.directory}"
     */
    private File projectBuildDirectory;

    // checked 20110331
    @Override
    public void execute () throws MojoExecutionException {
        try {
            getLog ().info ("start weaving " + getRootPathToBeWoven ());
            new ClassFolderWeaver (getProjectClasspathElements ()).weave (getRootPathToBeWoven (), createWeavingProgressListener ());
            getLog ().info ("end   weaving " + getRootPathToBeWoven ());
        }
        catch (IOException e) {
            throw new MojoExecutionException (e.getMessage (), e);
        }
        catch (IllegalUseOfConstraintAnnotationException e) {
            throw new MojoExecutionException (e.getMessage (), e);
        }
    }

    // checked 20110331

    /**
     * Getter for sub classes.
     *
     * @return projectBuildDirectory
     */
    protected File getProjectBuildDirectory () {
        return projectBuildDirectory;
    }

    // checked 20110331

    /**
     * Root path to be woven.
     */
    @NotEmpty
    abstract protected String getRootPathToBeWoven ();

    // checked 20110331

    /**
     * Returns classpaths
     *
     * @return classpaths
     */
    abstract protected String[] getProjectClasspathElements ();

    // checked 20110331
    private WeavingProgressListener createWeavingProgressListener () {
        return new WeavingProgressListener() {
            @Override
            public void notified (File classFile, NotClassFileException e) {
                getLog ().debug (" - skip : " + classFile.getAbsolutePath ());
            }

            @Override
            public void notified (File classFile, NoWeavingNeededException e) {
                getLog ().debug (" - no weaving needed : " + classFile.getAbsolutePath ());
            }

            @Override
            public void notified (File classFile, IllegalUseOfConstraintAnnotationException e) {
                // TODO 2011-03-31: ログメッセージを作りにあたって、e.getDetailedMessage() 側の文字列を変更される危険性を考えると、MethodInfo のような情報取得用 interface から情報を取得
                // して文字列を作成したほうが安全。実際、リビジョン463頃にはそうしていた。しかし現在は、e.getDetailedDescription() がそれなりの情報をそれなりに出していてくれている
                // ことと、そのメッセージはそうそうは変更されないだろうという希望的観測から、文字列をいじる方法を取っている。この方法で実際に問題が起こったらリビジョン463のような方法に
                // 戻すことを検討しましょう。
                String[] lines = e.getDetailedDescription ().split ("\\n");
                for (int i = 0; i < lines.length; i++) {
                    if (i == 0) {
                        getLog ().info (" - " + lines[i]);
                        getLog ().info ("   class  file -> " + classFile.getAbsolutePath ());
                    }
                    else {
                        getLog ().info ("   " + lines[i]);
                    }
                }
            }
        };
    }
}
